@@ -2,7 +2,9 @@
 //! virtual dom into a writable buffer
 //!
 use crate::html::attributes::SegregatedAttributes;
-use crate::{html::attributes, Attribute, Element, Node};
+use crate::vdom::Leaf;
+use crate::vdom::NodeTrait;
+use crate::{html::attributes, vdom::Attribute, vdom::Element, vdom::Node};
 use std::fmt;
 
 /// render node, elements to a writable buffer
@@ -67,10 +69,29 @@ impl<MSG> Render for Node<MSG> {
             Node::Element(element) => {
                 element.render_with_indent(buffer, indent, compressed)
             }
-            Node::Text(text) => {
-                write!(buffer, "{}", &text.text)
+            Node::Leaf(leaf) => {
+                leaf.render_with_indent(buffer, indent, compressed)
             }
-            Node::Comment(comment) => {
+        }
+    }
+}
+
+impl Render for Leaf {
+    fn render_with_indent(
+        &self,
+        buffer: &mut dyn fmt::Write,
+        _indent: usize,
+        _compressed: bool,
+    ) -> fmt::Result {
+        match self {
+            Leaf::Text(text) => {
+                write!(buffer, "{}", text)
+            }
+            Leaf::SafeHtml(html) => {
+                //TODO: html escape this one
+                write!(buffer, "{}", html)
+            }
+            Leaf::Comment(comment) => {
                 write!(buffer, "<!--{}-->", comment)
             }
         }
